@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,42 +29,40 @@
     $Id$
 */
 
-/// \file builtin_layout.cc
-#define LOG_FAC log_facility_t::layout
+/// @file content/layout/builtin_layout.cc
+#define GRB_LOG_FAC GrbLogFacility::layout
 
 #include "builtin_layout.h" // API
 
-#include <regex>
-
 #include "cds/cds_container.h"
-#include "cds/cds_item.h"
 #include "config/config.h"
+#include "config/config_val.h"
 #include "config/result/autoscan.h"
 #include "config/result/box_layout.h"
-#include "content/content_manager.h"
+#include "content/content.h"
+#include "context.h"
 #include "metadata/metadata_enums.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 
 #ifdef ONLINE_SERVICES
-
 #include "content/onlineservice/online_service.h"
-
-#ifdef ATRAILERS
-#include "content/onlineservice/atrailers_content_handler.h"
 #endif
 
-#endif // ONLINE_SERVICES
+#include <regex>
 
-BuiltinLayout::BuiltinLayout(std::shared_ptr<ContentManager> content)
+BuiltinLayout::BuiltinLayout(std::shared_ptr<Content> content)
     : Layout(std::move(content))
     , config(this->content->getContext()->getConfig())
-    , genreMap(this->config->getDictionaryOption(CFG_IMPORT_SCRIPTING_IMPORT_GENRE_MAP))
+    , converterManager(this->content->getContext()->getConverterManager())
+    , genreMap(this->config->getDictionaryOption(ConfigVal::IMPORT_SCRIPTING_IMPORT_GENRE_MAP))
 {
-    auto blOption = config->getBoxLayoutListOption(CFG_BOXLAYOUT_BOX);
-    for (auto&& bl : blOption->getArrayCopy()) {
+    auto blOption = config->getBoxLayoutListOption(ConfigVal::BOXLAYOUT_LIST);
+    for (auto&& bl : EDIT_CAST(EditHelperBoxLayout, blOption)->getArrayCopy()) {
         // We could copy only the enabled containers, but to avoid checking dependencies here, we copy all of them!
         container[bl->getKey()] = std::make_shared<CdsContainer>(bl->getTitle());
+        if (!bl->getUpnpShortcut().empty())
+            container[bl->getKey()]->setUpnpShortcut(bl->getUpnpShortcut());
     }
 
     containerAt(BoxKeys::videoRoot)->addMetaData(MetadataFields::M_CONTENT_CLASS, UPNP_CLASS_VIDEO_ITEM);
@@ -79,20 +77,18 @@ BuiltinLayout::BuiltinLayout(std::shared_ptr<ContentManager> content)
 
     containerAt(BoxKeys::audioRoot)->addMetaData(MetadataFields::M_CONTENT_CLASS, UPNP_CLASS_AUDIO_ITEM);
     chain["/Audio/All Audio"] = this->content->addContainerTree({ containerAt(BoxKeys::audioRoot), containerAt(BoxKeys::audioAll) }, nullptr);
-    if (blOption->get(BoxKeys::audioAllTracks)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllTracks)->getEnabled()) {
         chain["/Audio/All - full name"] = this->content->addContainerTree({ containerAt(BoxKeys::audioRoot), containerAt(BoxKeys::audioAllTracks) }, nullptr);
     }
 
 #ifdef ONLINE_SERVICES
-    if (config->getBoolOption(CFG_ONLINE_CONTENT_ATRAILERS_ENABLED)) {
-#ifdef ATRAILERS
-        chain["/Online Services/Apple/All Trailers"] = this->content->addContainerTree({ containerAt(BoxKeys::trailerRoot), containerAt(BoxKeys::trailerApple), containerAt(BoxKeys::trailerAll) }, nullptr);
-#endif
-    }
 #endif
 }
 
-void BuiltinLayout::add(const std::shared_ptr<CdsObject>& obj, const std::pair<int, bool>& parentID, bool useRef)
+int BuiltinLayout::add(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::pair<int, bool>& parentID,
+    bool useRef)
 {
     obj->setParentID(parentID.first);
     if (useRef)
@@ -100,22 +96,32 @@ void BuiltinLayout::add(const std::shared_ptr<CdsObject>& obj, const std::pair<i
     obj->setID(INVALID_OBJECT_ID);
 
     content->addObject(obj, parentID.second);
+    return obj->getID();
 }
 
-void BuiltinLayout::getDir(const std::shared_ptr<CdsObject>& obj, const fs::path& rootPath, const std::string_view& c1, const std::string_view& c2, const std::string& upnpClass)
+int BuiltinLayout::getDir(
+    const std::shared_ptr<CdsObject>& obj,
+    const fs::path& rootPath,
+    BoxKeys c1,
+    BoxKeys c2,
+    const std::string& upnpClass)
 {
     fs::path dir;
     auto&& objPath = obj->getLocation();
     if (!rootPath.empty()) {
         // make location relative to rootpath: "/home/.../Videos/Action/a.mkv" with rootpath "/home/.../Videos" -> "Action"
-        dir = fs::relative(objPath.parent_path(), config->getBoolOption(CFG_IMPORT_LAYOUT_PARENT_PATH) ? rootPath.parent_path() : rootPath);
+        dir = fs::relative(objPath.parent_path(), config->getBoolOption(ConfigVal::IMPORT_LAYOUT_PARENT_PATH) ? rootPath.parent_path() : rootPath);
         if (dir == ".")
             dir = getLastPath(objPath);
     } else
         dir = getLastPath(objPath);
 
-    auto f2i = StringConverter::f2i(config);
-    dir = f2i->convert(dir);
+    auto f2i = converterManager->f2i();
+    auto [mval, err] = f2i->convert(dir);
+    if (!err.empty()) {
+        log_warning("{}: {}", dir.string(), err);
+    }
+    dir = mval;
 
     if (!dir.empty()) {
         std::vector<std::shared_ptr<CdsObject>> dirVect;
@@ -127,23 +133,29 @@ void BuiltinLayout::getDir(const std::shared_ptr<CdsObject>& obj, const fs::path
         }
         auto id = content->addContainerTree(dirVect, obj);
         dirVect[dirVect.size() - 1]->setClass(upnpClass);
-        add(obj, id);
+        return add(obj, id);
     }
+    return INVALID_OBJECT_ID;
 }
 
-void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addVideo(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsContainer>& parent,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
-    auto f2i = StringConverter::f2i(config);
-    auto blOption = config->getBoxLayoutListOption(CFG_BOXLAYOUT_BOX);
+    auto f2i = converterManager->f2i();
+    auto blOption = config->getBoxLayoutListOption(ConfigVal::BOXLAYOUT_LIST);
+    std::vector<int> result;
 
     auto id = chain["/Video/All Video"];
     log_debug("add video file {}", obj->getLocation().string());
 
     if (obj->getID() != INVALID_OBJECT_ID) {
         obj->setRefID(obj->getID());
-        add(obj, id);
+        result.push_back(add(obj, id));
     } else {
-        add(obj, id);
+        result.push_back(add(obj, id));
         obj->setRefID(obj->getID());
     }
 
@@ -162,7 +174,7 @@ void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const std::s
                 month.resize(m);
         }
 
-        if (blOption->get(BoxKeys::videoAllYears)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::videoAllYears)->getEnabled()) {
             if ((y > 0) && (m > 0)) {
                 std::vector<std::shared_ptr<CdsObject>> ct;
                 ct.reserve(4);
@@ -171,11 +183,11 @@ void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const std::s
                 ct.push_back(std::make_shared<CdsContainer>(year));
                 ct.push_back(std::make_shared<CdsContainer>(month));
                 id = content->addContainerTree(ct, obj);
-                add(obj, id);
+                result.push_back(add(obj, id));
             }
         }
 
-        if (blOption->get(BoxKeys::videoAllDates)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::videoAllDates)->getEnabled()) {
             auto t = date.find('T');
             if (t != std::string::npos) {
                 date.resize(t);
@@ -185,28 +197,34 @@ void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const std::s
             ct.push_back(containerAt(BoxKeys::videoAllDates));
             ct.push_back(std::make_shared<CdsContainer>(date));
             id = content->addContainerTree(ct, obj);
-            add(obj, id);
+            result.push_back(add(obj, id));
         }
     }
 
-    if (blOption->get(BoxKeys::videoAllDirectories)->getEnabled()) {
-        getDir(obj, rootpath, BoxKeys::videoRoot, BoxKeys::videoAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Video, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Video)));
+    if (blOption->getKey(BoxKeys::videoAllDirectories)->getEnabled()) {
+        result.push_back(getDir(obj, rootpath, BoxKeys::videoRoot, BoxKeys::videoAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Video, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Video))));
     }
+    return result;
 }
 
-void BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addImage(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsContainer>& parent,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
-    auto f2i = StringConverter::f2i(config);
-    auto blOption = config->getBoxLayoutListOption(CFG_BOXLAYOUT_BOX);
+    auto f2i = converterManager->f2i();
+    auto blOption = config->getBoxLayoutListOption(ConfigVal::BOXLAYOUT_LIST);
+    std::vector<int> result;
 
     log_debug("add image file {}", obj->getLocation().string());
 
     auto id = chain["/Photos/All Photos"];
     if (obj->getID() != INVALID_OBJECT_ID) {
         obj->setRefID(obj->getID());
-        add(obj, id);
+        result.push_back(add(obj, id));
     } else {
-        add(obj, id);
+        result.push_back(add(obj, id));
         obj->setRefID(obj->getID());
     }
 
@@ -225,7 +243,7 @@ void BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const std::s
                 month.resize(m);
         }
 
-        if (blOption->get(BoxKeys::imageAllYears)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::imageAllYears)->getEnabled()) {
             if ((y > 0) && (m > 0)) {
                 std::vector<std::shared_ptr<CdsObject>> ct;
                 ct.push_back(containerAt(BoxKeys::imageRoot));
@@ -233,33 +251,39 @@ void BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const std::s
                 ct.push_back(std::make_shared<CdsContainer>(year));
                 ct.push_back(std::make_shared<CdsContainer>(month));
                 id = content->addContainerTree(ct, obj);
-                add(obj, id);
+                result.push_back(add(obj, id));
             }
         }
 
-        if (blOption->get(BoxKeys::imageAllDates)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::imageAllDates)->getEnabled()) {
             auto t = date.find('T');
             if (t != std::string::npos) {
                 date.resize(t);
             }
             std::vector<std::shared_ptr<CdsObject>> ct;
             ct.push_back(containerAt(BoxKeys::imageRoot));
-            ct.push_back(containerAt(BoxKeys::imageAllYears));
+            ct.push_back(containerAt(BoxKeys::imageAllDates));
             ct.push_back(std::make_shared<CdsContainer>(date));
             id = content->addContainerTree(ct, obj);
-            add(obj, id);
+            result.push_back(add(obj, id));
         }
     }
 
-    if (blOption->get(BoxKeys::imageAllDirectories)->getEnabled()) {
-        getDir(obj, rootpath, BoxKeys::imageRoot, BoxKeys::imageAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Image, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Image)));
+    if (blOption->getKey(BoxKeys::imageAllDirectories)->getEnabled()) {
+        result.push_back(getDir(obj, rootpath, BoxKeys::imageRoot, BoxKeys::imageAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Image, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Image))));
     }
+    return result;
 }
 
-void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addAudio(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsContainer>& parent,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
-    auto f2i = StringConverter::f2i(config);
-    auto blOption = config->getBoxLayoutListOption(CFG_BOXLAYOUT_BOX);
+    auto f2i = converterManager->f2i();
+    auto blOption = config->getBoxLayoutListOption(ConfigVal::BOXLAYOUT_LIST);
+    std::vector<int> result;
 
     std::string desc;
     log_debug("add audio file {}", obj->getLocation().string());
@@ -331,24 +355,27 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const std::s
     // id
     if (obj->getID() != INVALID_OBJECT_ID) {
         obj->setRefID(obj->getID());
-        add(obj, id);
+        result.push_back(add(obj, id));
     } else {
         // the object is not yet in the database (probably we got it from a
         // playlist script, so we set the ref id after adding - it will be used
         // for all consequent virtual objects
-        add(obj, id);
+        result.push_back(add(obj, id));
         obj->setRefID(obj->getID());
     }
 
     auto artistContainer = std::make_shared<CdsContainer>(artist, UPNP_CLASS_MUSIC_ARTIST);
-    if (blOption->get(BoxKeys::audioAllSongs)->getEnabled() && blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+    artistContainer->addMetaData(MetadataFields::M_ARTIST, artist);
+    artistContainer->addMetaData(MetadataFields::M_ALBUMARTIST, artist);
+    artistContainer->addMetaData(MetadataFields::M_GENRE, genre);
+    if (blOption->getKey(BoxKeys::audioAllSongs)->getEnabled() && blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
         std::vector<std::shared_ptr<CdsObject>> arc;
         arc.push_back(containerAt(BoxKeys::audioRoot));
         arc.push_back(containerAt(BoxKeys::audioAllArtists));
         arc.push_back(artistContainer);
         arc.push_back(containerAt(BoxKeys::audioAllSongs));
         id = content->addContainerTree(arc, obj);
-        add(obj, id);
+        result.push_back(add(obj, id));
     }
 
     std::string prefixTitle;
@@ -362,66 +389,73 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const std::s
 
     auto albumContainer = std::make_shared<CdsContainer>(album, UPNP_CLASS_MUSIC_ALBUM);
     albumContainer->setMetaData(obj->getMetaData());
-    albumContainer->setResources(obj->getResources());
+    if (parent && parent->getResourceCount() > 0)
+        albumContainer->setResources(parent->getResources());
     albumContainer->setRefID(obj->getID());
-    artistContainer->setSearchable(true);
+    artistContainer->setSearchable(blOption->getKey(BoxKeys::audioAllArtists)->getSearchable());
 
-    if (blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
         std::vector<std::shared_ptr<CdsObject>> alc;
         alc.push_back(containerAt(BoxKeys::audioRoot));
         alc.push_back(containerAt(BoxKeys::audioAllArtists));
         alc.push_back(artistContainer);
         alc.push_back(albumContainer);
         id = content->addContainerTree(alc, obj);
-        add(obj, id);
+        result.push_back(add(obj, id));
     }
 
-    albumContainer->setSearchable(true);
-    if (blOption->get(BoxKeys::audioAllAlbums)->getEnabled()) {
+    albumContainer->setSearchable(blOption->getKey(BoxKeys::audioAllAlbums)->getSearchable());
+    if (blOption->getKey(BoxKeys::audioAllAlbums)->getEnabled()) {
         std::vector<std::shared_ptr<CdsObject>> allc;
         allc.push_back(containerAt(BoxKeys::audioRoot));
         allc.push_back(containerAt(BoxKeys::audioAllAlbums));
         allc.push_back(std::move(albumContainer));
         id = content->addContainerTree(allc, obj);
-        add(obj, id);
+        result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllGenres)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllGenres)->getEnabled()) {
+        auto genreContainer = std::make_shared<CdsContainer>(genre, UPNP_CLASS_MUSIC_GENRE);
+        genreContainer->addMetaData(MetadataFields::M_GENRE, genre);
+        genreContainer->setSearchable(blOption->getKey(BoxKeys::audioAllGenres)->getSearchable());
         std::vector<std::shared_ptr<CdsObject>> ct;
         ct.push_back(containerAt(BoxKeys::audioRoot));
         ct.push_back(containerAt(BoxKeys::audioAllGenres));
-        ct.push_back(std::make_shared<CdsContainer>(genre, UPNP_CLASS_MUSIC_GENRE));
+        ct.push_back(std::move(genreContainer));
         id = content->addContainerTree(ct, obj);
-        add(obj, id);
+        result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllComposers)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllComposers)->getEnabled()) {
         auto composerContainer = std::make_shared<CdsContainer>(composer, UPNP_CLASS_MUSIC_COMPOSER);
+        composerContainer->addMetaData(MetadataFields::M_COMPOSER, composer);
         composerContainer->setSearchable(true);
         std::vector<std::shared_ptr<CdsObject>> cc;
         cc.push_back(containerAt(BoxKeys::audioRoot));
         cc.push_back(containerAt(BoxKeys::audioAllComposers));
         cc.push_back(std::move(composerContainer));
         id = content->addContainerTree(cc, obj);
-        add(obj, id);
+        result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllYears)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllYears)->getEnabled()) {
         auto yearContainer = std::make_shared<CdsContainer>(date);
+        yearContainer->addMetaData(MetadataFields::M_DATE, date);
+        yearContainer->addMetaData(MetadataFields::M_UPNP_DATE, date);
         yearContainer->setSearchable(true);
         std::vector<std::shared_ptr<CdsObject>> yt;
         yt.push_back(containerAt(BoxKeys::audioRoot));
         yt.push_back(containerAt(BoxKeys::audioAllYears));
         yt.push_back(std::move(yearContainer));
         id = content->addContainerTree(yt, obj);
-        add(obj, id);
+        result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllDirectories)->getEnabled()) {
-        getDir(obj, rootpath, BoxKeys::audioRoot, BoxKeys::audioAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Audio, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Audio)));
+    if (blOption->getKey(BoxKeys::audioAllDirectories)->getEnabled()) {
+        result.push_back(getDir(obj, rootpath, BoxKeys::audioRoot, BoxKeys::audioAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Audio, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Audio))));
     }
 
-    if (blOption->get(BoxKeys::audioArtistChronology)->getEnabled() && blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioArtistChronology)->getEnabled() && blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
         artistContainer->setSearchable(false);
         std::vector<std::shared_ptr<CdsObject>> chronology;
         chronology.push_back(containerAt(BoxKeys::audioRoot));
@@ -430,107 +464,42 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const std::s
         chronology.push_back(containerAt(BoxKeys::audioArtistChronology));
         chronology.push_back(std::make_shared<CdsContainer>(date + " - " + album, UPNP_CLASS_MUSIC_ALBUM));
         id = content->addContainerTree(chronology, obj);
-        add(obj, id);
+        result.push_back(add(obj, id));
     }
 
     // Keep this last, since it's modifying the object title
-    if (blOption->get(BoxKeys::audioAllTracks)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllTracks)->getEnabled()) {
         artistContainer->setSearchable(true);
         obj->setTitle(fmt::format("{}{}", prefixTitle, title));
-        add(obj, chain["/Audio/All - full name"]);
+        result.push_back(add(obj, chain["/Audio/All - full name"]));
 
-        if (blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
             std::vector<std::shared_ptr<CdsObject>> all;
             all.push_back(containerAt(BoxKeys::audioRoot));
             all.push_back(containerAt(BoxKeys::audioAllArtists));
             all.push_back(std::move(artistContainer));
             all.push_back(containerAt(BoxKeys::audioAllTracks));
             id = content->addContainerTree(all, obj);
-            add(obj, id);
+            result.push_back(add(obj, id));
         }
     }
+    return result;
 }
 
 #ifdef ONLINE_SERVICES
-void BuiltinLayout::addTrailer(const std::shared_ptr<CdsObject>& obj, OnlineServiceType serviceType, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addOnlineItem(
+    const std::shared_ptr<CdsObject>& obj,
+    OnlineServiceType serviceType,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     switch (serviceType) {
-#ifdef ATRAILERS
-    case OnlineServiceType::OS_ATrailers:
-        addATrailers(obj);
-        break;
-#endif
-    case OnlineServiceType::OS_Max:
+    case OnlineServiceType::Max:
     default:
         log_warning("No handler for service type");
         break;
     }
-}
-#endif
-
-#ifdef ATRAILERS
-void BuiltinLayout::addATrailers(const std::shared_ptr<CdsObject>& obj)
-{
-    auto blOption = config->getBoxLayoutListOption(CFG_BOXLAYOUT_BOX);
-
-    log_debug("add trailer {}", obj->getLocation().string());
-    {
-        auto id = chain["/Online Services/Apple/All Trailers"];
-        if (obj->getID() != INVALID_OBJECT_ID) {
-            obj->setRefID(obj->getID());
-            add(obj, id);
-        } else {
-            add(obj, id);
-            obj->setRefID(obj->getID());
-        }
-    }
-
-    auto meta = obj->getMetaData();
-
-    if (blOption->get(BoxKeys::trailerAllGenres)->getEnabled() && blOption->get(BoxKeys::trailerApple)->getEnabled()) {
-        std::string temp = getValueOrDefault(meta, MetaEnumMapper::getMetaFieldName(MetadataFields::M_GENRE));
-        auto genreAr = splitString(temp, ',');
-        for (auto&& genre : genreAr) {
-            trimStringInPlace(genre);
-            genre = mapGenre(genre);
-            if (genre.empty())
-                continue;
-
-            std::vector<std::shared_ptr<CdsObject>> ct;
-            ct.push_back(containerAt(BoxKeys::trailerRoot));
-            ct.push_back(containerAt(BoxKeys::trailerApple));
-            ct.push_back(containerAt(BoxKeys::trailerAllGenres));
-            ct.push_back(std::make_shared<CdsContainer>(genre));
-            auto id = content->addContainerTree(ct, obj);
-            add(obj, id);
-        }
-    }
-
-    if (blOption->get(BoxKeys::trailerRelDate)->getEnabled() && blOption->get(BoxKeys::trailerApple)->getEnabled()) {
-        std::string temp = getValueOrDefault(meta, MetaEnumMapper::getMetaFieldName(MetadataFields::M_DATE));
-        if (temp.length() >= 7) {
-            std::vector<std::shared_ptr<CdsObject>> ct;
-            ct.push_back(containerAt(BoxKeys::trailerRoot));
-            ct.push_back(containerAt(BoxKeys::trailerApple));
-            ct.push_back(containerAt(BoxKeys::trailerRelDate));
-            ct.push_back(std::make_shared<CdsContainer>(temp.substr(0, 7)));
-            auto id = content->addContainerTree(ct, obj);
-            add(obj, id);
-        }
-    }
-
-    if (blOption->get(BoxKeys::trailerPostDate)->getEnabled() && blOption->get(BoxKeys::trailerApple)->getEnabled()) {
-        std::string temp = obj->getAuxData(ATRAILERS_AUXDATA_POST_DATE);
-        if (temp.length() >= 7) {
-            std::vector<std::shared_ptr<CdsObject>> ct;
-            ct.push_back(containerAt(BoxKeys::trailerRoot));
-            ct.push_back(containerAt(BoxKeys::trailerApple));
-            ct.push_back(containerAt(BoxKeys::trailerPostDate));
-            ct.push_back(std::make_shared<CdsContainer>(temp.substr(0, 7)));
-            auto id = content->addContainerTree(ct, obj);
-            add(obj, id);
-        }
-    }
+    return {};
 }
 #endif
 
@@ -543,3 +512,8 @@ std::string BuiltinLayout::mapGenre(const std::string& genre)
     }
     return genre;
 }
+
+std::shared_ptr<CdsContainer> BuiltinLayout::containerAt(BoxKeys boxKey)
+{
+    return container.at(BoxLayout::getBoxKey(boxKey));
+}
