@@ -29,8 +29,8 @@
     $Id$
 */
 
-/// \file conn_mgr_service.h
-/// \brief Definition of the ConnectionManagerService class.
+/// @file upnp/conn_mgr_service.h
+/// @brief Definition of the ConnectionManagerService class.
 #ifndef __UPNP_CM_H__
 #define __UPNP_CM_H__
 
@@ -40,21 +40,21 @@ class CdsObject;
 class Context;
 class Database;
 
-/// \brief This class is responsible for the UPnP Connection Manager Service operations.
+/// @brief This class is responsible for the UPnP Connection Manager Service operations.
 ///
 /// Handles subscription and action invocation requests for the Connection Manager.
 class ConnectionManagerService : public UpnpService {
 protected:
-    /// \brief UPnP standard defined action: GetCurrentConnectionIDs()
-    /// \param request Incoming ActionRequest.
+    /// @brief UPnP standard defined action: GetCurrentConnectionIDs()
+    /// @param request Incoming ActionRequest.
     ///
     /// GetCurrentConnectionIDs(string ConnectionIDs)
     ///
     /// This is currently unsupported (returns empty string)
     void doGetCurrentConnectionIDs(ActionRequest& request) const;
 
-    /// \brief UPnP standard defined action: GetCurrentConnectionInfo()
-    /// \param request Incoming ActionRequest.
+    /// @brief UPnP standard defined action: GetCurrentConnectionInfo()
+    /// @param request Incoming ActionRequest.
     ///
     /// GetCurrentConnectionInfo(i4 ConnectoinID, i4 RcsID, i4 AVTransportID, string ProtocolInfo,
     /// string PeerConnectionManager, i4 PeerConnectionID, string Direction, string Status)
@@ -62,8 +62,8 @@ protected:
     /// This action is currently unsupported.
     void doGetCurrentConnectionInfo(ActionRequest& request) const;
 
-    /// \brief UPnP standard defined action: GetProtocolInfo()
-    /// \param request Incoming ActionRequest.
+    /// @brief UPnP standard defined action: GetProtocolInfo()
+    /// @param request Incoming ActionRequest.
     ///
     /// GetProtocolInfo(string Source, string Sink)
     void doGetProtocolInfo(ActionRequest& request) const;
@@ -71,20 +71,20 @@ protected:
     std::shared_ptr<Database> database;
 
 public:
-    /// \brief Constructor for the CMS, saves the service type and service id
+    /// @brief Constructor for the CMS, saves the service type and service id
     /// in internal variables.
     explicit ConnectionManagerService(const std::shared_ptr<Context>& context,
         const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder, UpnpDevice_Handle deviceHandle);
 
-    /// \brief Processes an incoming SubscriptionRequest.
-    /// \param request Incoming SubscriptionRequest.
+    /// @brief Processes an incoming SubscriptionRequest.
+    /// @param request Incoming SubscriptionRequest.
     ///
     /// Looks at the incoming SubscriptionRequest and accepts the subscription
     /// if everything is ok.
-    bool processSubscriptionRequest(const SubscriptionRequest& request) override;
+    bool processSubscriptionRequest(const SubscriptionRequest& request) const override;
 
-    /// \brief Sends out an event to all subscribed devices.
-    /// \param sourceProtocolCsv Comma Separated Value list of protocol information
+    /// @brief Sends out an event to all subscribed devices.
+    /// @param sourceProtocolCsv Comma Separated Value list of protocol information
     ///
     /// Sends out an update with protocol information to all subscribed devices
     bool sendSubscriptionUpdate(const std::string& sourceProtocolCsv) override;
